@@ -96,26 +96,31 @@ class RawMasterMixin(Model):
         if self.is_initial_cqrs_save and (not self._state.adding):
             self.cqrs_revision = F('cqrs_revision') + 1
 
-        self._save_tracked_fields()
+        self.save_tracked_fields()
 
         return super(RawMasterMixin, self).save(*args, **kwargs)
 
-    def _save_tracked_fields(self):
+    def save_tracked_fields(self):
         if hasattr(self, FIELDS_TRACKER_FIELD_NAME):
             tracker = getattr(self, FIELDS_TRACKER_FIELD_NAME)
-            setattr(self, TRACKED_FIELDS_ATTR_NAME, tracker.changed())
+            if self._state.adding:
+                data = tracker.changed_initial()
+            else:
+                data = tracker.changed()
+            setattr(self, TRACKED_FIELDS_ATTR_NAME, data)
 
-    def to_cqrs_dict(self, using=None):
+    def to_cqrs_dict(self, using=None, sync=False):
         """CQRS serialization for transport payload.
 
         :param using: The using argument can be used to force the database
                       to use, defaults to None
         :type using: str, optional
+        :type sync: bool, optional
         :return: The serialized instance data.
         :rtype: dict
         """
         if self.CQRS_SERIALIZER:
-            data = self._class_serialization(using)
+            data = self._class_serialization(using, sync=sync)
         else:
             self._refresh_f_expr_values(using)
             data = self._common_serialization(using)
@@ -181,15 +186,18 @@ class RawMasterMixin(Model):
         """
         return queryset
 
+    def get_custom_cqrs_delete_data(self):
+        """ This method should be overridden when additional data is needed in DELETE payload. """
+        pass
+
     @classmethod
     def call_post_bulk_create(cls, instances, using=None):
         """ Post bulk create signal caller (django doesn't support it by default).
 
         .. code-block:: python
 
-            # On PostgreSQL
-            instances = model.objects.bulk_create(instances)
-            model.call_post_bulk_create(instances)
+            # Used automatically by cqrs.bulk_create()
+            instances = model.cqrs.bulk_create(instances)
         """
         post_bulk_create.send(cls, instances=instances, using=using)
 
@@ -230,13 +238,16 @@ class RawMasterMixin(Model):
 
         return data
 
-    def _class_serialization(self, using):
-        db = using if using is not None else self._state.db
-        qs = self.__class__._default_manager.using(db).filter(pk=self.pk)
+    def _class_serialization(self, using, sync=False):
+        if sync:
+            instance = self
+        else:
+            db = using if using is not None else self._state.db
+            qs = self.__class__._default_manager.using(db).filter(pk=self.pk)
 
-        instance = self.relate_cqrs_serialization(qs).first()
-        if not instance:
-            raise RuntimeError("Couldn't serialize CQRS class ({}).".format(self.CQRS_ID))
+            instance = self.relate_cqrs_serialization(qs).first()
+            if not instance:
+                raise RuntimeError("Couldn't serialize CQRS class ({}).".format(self.CQRS_ID))
 
         data = self._cqrs_serializer_cls(instance).data
         data['cqrs_revision'] = instance.cqrs_revision
@@ -264,8 +275,8 @@ class RawMasterMixin(Model):
             if value is not None and isinstance(value, CombinedExpression):
                 fields_to_refresh.append(f.name)
 
-            if fields_to_refresh:
-                self.refresh_from_db(fields=fields_to_refresh)
+        if fields_to_refresh:
+            self.refresh_from_db(fields=fields_to_refresh)
 
     @property
     def _cqrs_serializer_cls(self):
