@@ -6,7 +6,6 @@ import time
 from socket import gaierror
 from urllib.parse import unquote, urlparse
 
-
 import ujson
 from django.conf import settings
 from pika import exceptions, BasicProperties, BlockingConnection, ConnectionParameters, credentials
@@ -24,6 +23,21 @@ logger = logging.getLogger('django-cqrs')
 class RabbitMQTransport(LoggingMixin, BaseTransport):
     CONSUMER_RETRY_TIMEOUT = 5
 
+    _producer_connection = None
+    _producer_channel = None
+
+    @classmethod
+    def clean_connection(cls):
+        connection = cls._producer_connection
+        if connection and not connection.is_closed:
+            try:
+                connection.close()
+            except (exceptions.StreamLostError, exceptions.ConnectionClosed, ConnectionError):
+                logger.warning("Connection was closed or is closing. Skip it...")
+
+        cls._producer_connection = None
+        cls._producer_channel = None
+
     @classmethod
     def consume(cls):
         consumer_rabbit_settings = cls._get_consumer_settings()
@@ -43,28 +57,39 @@ class RabbitMQTransport(LoggingMixin, BaseTransport):
                 logger.error('AMQP connection error. Reconnecting...')
                 time.sleep(cls.CONSUMER_RETRY_TIMEOUT)
             finally:
-                if connection:
+                if connection and not connection.is_closed:
                     connection.close()
 
     @classmethod
     def produce(cls, payload):
-        rmq_settings = cls._get_common_settings()
-        exchange = rmq_settings[-1]
-
-        connection = None
         try:
-            # Decided not to create context-manager to stay within the class
-            connection, channel = cls._get_producer_rmq_objects(*rmq_settings)
-
-            cls._produce_message(channel, exchange, payload)
-            cls.log_produced(payload)
+            cls._produce(payload)
         except (exceptions.AMQPError, exceptions.ChannelError, exceptions.ReentrancyError):
-            logger.error("CQRS couldn't be published: pk = {} ({}).".format(
+            logger.error("CQRS couldn't be published: pk = {} ({}). Reconnect...".format(
                 payload.pk, payload.cqrs_id,
             ))
-        finally:
-            if connection:
-                connection.close()
+
+            # in case of any error - close connection and try to reconnect
+            cls.clean_connection()
+            # reconnect at least 1 time
+            try:
+                cls._produce(payload)
+            except (exceptions.AMQPError, exceptions.ChannelError, exceptions.ReentrancyError):
+                logger.error("CQRS couldn't be published: pk = {} ({}).".format(
+                    payload.pk, payload.cqrs_id,
+                ))
+
+                cls.clean_connection()
+
+    @classmethod
+    def _produce(cls, payload):
+        rmq_settings = cls._get_common_settings()
+        exchange = rmq_settings[-1]
+        # Decided not to create context-manager to stay within the class
+        _, channel = cls._get_producer_rmq_objects(*rmq_settings, signal_type=payload.signal_type)
+
+        cls._produce_message(channel, exchange, payload)
+        cls.log_produced(payload)
 
     @classmethod
     def _consume_message(cls, ch, method, properties, body):
@@ -99,7 +124,7 @@ class RabbitMQTransport(LoggingMixin, BaseTransport):
             ch.basic_ack(delivery_tag=method.delivery_tag)
             cls.log_consumed_accepted(payload)
         else:
-            ch.basic_nack(delivery_tag=method.delivery_tag, requeue=False)
+            ch.basic_nack(delivery_tag=method.delivery_tag)
             cls.log_consumed_denied(payload)
 
     @classmethod
@@ -111,7 +136,11 @@ class RabbitMQTransport(LoggingMixin, BaseTransport):
             routing_key=routing_key,
             body=ujson.dumps(payload.to_dict()),
             mandatory=True,
-            properties=BasicProperties(content_type='text/plain', delivery_mode=2)
+            properties=BasicProperties(
+                content_type='text/plain',
+                delivery_mode=2,  # make message persistent
+                expiration=settings.CQRS.get('MESSAGE_TTL', '60000'),  # milliseconds
+            )
         )
 
     @staticmethod
@@ -154,7 +183,24 @@ class RabbitMQTransport(LoggingMixin, BaseTransport):
         return connection, channel
 
     @classmethod
-    def _get_producer_rmq_objects(cls, host, port, creds, exchange):
+    def _get_producer_rmq_objects(cls, host, port, creds, exchange, signal_type=None):
+        """
+        Use shared connection in case of sync mode, otherwise create new connection for each
+        message
+        """
+        if signal_type == SignalType.SYNC:
+            if cls._producer_connection is None:
+                connection, channel = cls._create_connection(host, port, creds, exchange)
+
+                cls._producer_connection = connection
+                cls._producer_channel = channel
+
+            return cls._producer_connection, cls._producer_channel
+        else:
+            return cls._create_connection(host, port, creds, exchange)
+
+    @classmethod
+    def _create_connection(cls, host, port, creds, exchange):
         connection = BlockingConnection(
             ConnectionParameters(
                 host=host,
